import { getFileContentAndPath } from '../../../bin/frameworks/filesystem/getFileContentAndPath';

import {
  getFileContentAndPathOperationToken,
  getFileContentAndPathOperationComponent,
  getFileContentAndPathOperationStyle,
  getFileContentAndPathOperationCss,
  getFileContentAndPathOperationStory,
  getFileContentAndPathOperationDescription
} from '../../../testdata/getFileContentAndPathOperation';

import {
  ErrorGetFileContentAndPath,
  ErrorGetFileContentAndPathMissingFields,
  ErrorGetFileContentAndPathNoReturn
} from '../../../bin/frameworks/errors/errors';

describe('Failure cases', () => {
  test('It should throw an error if running without arguments', () => {
    // @ts-ignore
    expect(() => getFileContentAndPath()).toThrowError(ErrorGetFileContentAndPath);
  });

  test('It should throw an error if missing any fields', () => {
    // @ts-ignore
    expect(() => getFileContentAndPath({})).toThrowError(ErrorGetFileContentAndPathMissingFields);
  });

  test('It should throw an error if it cannot resolve a valid type', () => {
    const invalidContentType = {
      type: 1,
      file: 1,
      path: 1,
      name: 1,
      format: 1,
      element: 1
    };

    expect(() =>
      // @ts-ignore
      getFileContentAndPath(invalidContentType)
    ).toThrowError(ErrorGetFileContentAndPathNoReturn);
  });
});

describe('Success cases', () => {
  describe('Tokens', () => {
    test('It should return valid data for tokens', () => {
      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationToken)
      ).toMatchObject({
        fileContent: `// THIS FILE IS AUTO-GENERATED BY FIGMAGIC. DO NOT MAKE EDITS IN THIS FILE! CHANGES WILL GET OVER-WRITTEN BY ANY FURTHER PROCESSING.

const colors = {
 \"green3\": \"rgba(111, 207, 151, 1)\",
 \"green2\": \"rgba(39, 174, 96, 1)\",
 \"green1\": \"rgba(33, 150, 83, 1)\",
 \"blue3\": \"rgba(86, 204, 242, 1)\",
 \"blue2\": \"rgba(45, 156, 219, 1)\",
 \"blue1\": \"rgba(47, 128, 237, 1)\",
 \"yellow\": \"rgba(242, 201, 76, 1)\",
 \"orange\": \"rgba(242, 153, 74, 1)\",
 \"red\": \"rgba(235, 87, 87, 1)\",
 \"neon\": \"rgba(228, 255, 193, 1)\",
 \"gray5\": \"rgba(242, 242, 242, 1)\",
 \"gray4\": \"rgba(224, 224, 224, 1)\",
 \"gray3\": \"rgba(189, 189, 189, 1)\",
 \"gray2\": \"rgba(130, 130, 130, 1)\",
 \"gray1\": \"rgba(79, 79, 79, 1)\",
 \"white\": \"rgba(255, 255, 255, 1)\",
 \"black\": \"rgba(51, 51, 51, 1)\"
}

export default colors;`,
        filePath: 'tokens/colors.mjs'
      });
    });

    test('It should return valid data for tokens (ts)', () => {
      expect(
        // @ts-ignore
        getFileContentAndPath({ ...getFileContentAndPathOperationToken, format: 'ts' })
      ).toMatchObject({
        fileContent: `// THIS FILE IS AUTO-GENERATED BY FIGMAGIC. DO NOT MAKE EDITS IN THIS FILE! CHANGES WILL GET OVER-WRITTEN BY ANY FURTHER PROCESSING.

const colors = {
 \"green3\": \"rgba(111, 207, 151, 1)\",
 \"green2\": \"rgba(39, 174, 96, 1)\",
 \"green1\": \"rgba(33, 150, 83, 1)\",
 \"blue3\": \"rgba(86, 204, 242, 1)\",
 \"blue2\": \"rgba(45, 156, 219, 1)\",
 \"blue1\": \"rgba(47, 128, 237, 1)\",
 \"yellow\": \"rgba(242, 201, 76, 1)\",
 \"orange\": \"rgba(242, 153, 74, 1)\",
 \"red\": \"rgba(235, 87, 87, 1)\",
 \"neon\": \"rgba(228, 255, 193, 1)\",
 \"gray5\": \"rgba(242, 242, 242, 1)\",
 \"gray4\": \"rgba(224, 224, 224, 1)\",
 \"gray3\": \"rgba(189, 189, 189, 1)\",
 \"gray2\": \"rgba(130, 130, 130, 1)\",
 \"gray1\": \"rgba(79, 79, 79, 1)\",
 \"white\": \"rgba(255, 255, 255, 1)\",
 \"black\": \"rgba(51, 51, 51, 1)\"
} as const;

export default colors;`,
        filePath: 'tokens/colors.ts'
      });
    });

    test('It should return valid data for tokens (enum)', () => {
      const data = {
        ...getFileContentAndPathOperationToken,
        metadata: {
          dataType: 'enum'
        }
      };

      expect(
        // @ts-ignore
        getFileContentAndPath(data)
      ).toMatchObject({
        fileContent: `// THIS FILE IS AUTO-GENERATED BY FIGMAGIC. DO NOT MAKE EDITS IN THIS FILE! CHANGES WILL GET OVER-WRITTEN BY ANY FURTHER PROCESSING.

enum colors {
  'green3' = 'rgba(111, 207, 151, 1)',
  'green2' = 'rgba(39, 174, 96, 1)',
  'green1' = 'rgba(33, 150, 83, 1)',
  'blue3' = 'rgba(86, 204, 242, 1)',
  'blue2' = 'rgba(45, 156, 219, 1)',
  'blue1' = 'rgba(47, 128, 237, 1)',
  'yellow' = 'rgba(242, 201, 76, 1)',
  'orange' = 'rgba(242, 153, 74, 1)',
  'red' = 'rgba(235, 87, 87, 1)',
  'neon' = 'rgba(228, 255, 193, 1)',
  'gray5' = 'rgba(242, 242, 242, 1)',
  'gray4' = 'rgba(224, 224, 224, 1)',
  'gray3' = 'rgba(189, 189, 189, 1)',
  'gray2' = 'rgba(130, 130, 130, 1)',
  'gray1' = 'rgba(79, 79, 79, 1)',
  'white' = 'rgba(255, 255, 255, 1)',
  'black' = 'rgba(51, 51, 51, 1)',
}

export default colors;`,
        filePath: 'tokens/colors'
      });
    });
  });

  describe('Components', () => {
    // TODO: Test fails inexplicably on Windows?
    test('It should return valid data for components', () => {
      const fileContent = `import React from 'react';
import PropTypes from 'prop-types';

import colorsStyled from './colorsStyled';

const colors = (props) => <colorsStyled {...props}>{props.children ? props.children : \"text\"}</colorsStyled>;

colors.propTypes = {};

export default colors;`;

      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationComponent)
      ).toMatchObject({
        fileContent: fileContent,
        filePath: 'tokens/colors.mjs'
      });
    });
  });

  describe('Style', () => {
    test('It should return valid data for style', () => {
      const fileContent = `import styled from 'styled-components';

import colorsCss from './colorsCss';

// Extend the below as needed
const colorsStyled = styled.div\`
  \${colorsCss};
\`;

export default colorsStyled;`;

      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationStyle)
      ).toMatchObject({
        fileContent: fileContent,
        filePath: 'tokens/colorsStyled.mjs'
      });
    });
  });

  describe('CSS', () => {
    test('It should return valid data for CSS', () => {
      const fileContent = `// THIS FILE IS AUTO-GENERATED BY FIGMAGIC. DO NOT MAKE EDITS IN THIS FILE! CHANGES WILL GET OVER-WRITTEN BY ANY FURTHER PROCESSING.
\n
const colorsCss = \`[object Object]\`;

export default colorsCss;`;

      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationCss)
      ).toMatchObject({
        fileContent: fileContent,
        filePath: 'tokens/colorsCss.mjs'
      });
    });
  });

  describe('Story', () => {
    test('It should return valid data for Storybook', () => {
      const fileContent = `import React from 'react';
import colors from './colors';

import notes from './colors.description.md';

export default { title: 'colors', parameters: { notes } };

export const colorsRegular = () => <colors>text</colors>;`;

      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationStory)
      ).toMatchObject({
        fileContent: fileContent,
        filePath: 'tokens/colors.stories.js'
      });
    });
  });

  describe('Description', () => {
    test('It should return valid data for description', () => {
      const fileContent = `<!--THIS FILE IS AUTO-GENERATED BY FIGMAGIC. DO NOT MAKE EDITS IN THIS FILE! CHANGES WILL GET OVER-WRITTEN BY ANY FURTHER PROCESSING.-->
[object Object]`;

      expect(
        // @ts-ignore
        getFileContentAndPath(getFileContentAndPathOperationDescription)
      ).toMatchObject({
        fileContent: fileContent,
        filePath: 'tokens/colors.description.mjs'
      });
    });
  });
});
